﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SimpleCalculator
{
    public class CalcController : ICalcController
    {
        ICalcModel model;
        ICalcView view;

        public CalcController(ICalcModel calcModel, ICalcView CalcView)
        {
            model = calcModel;
            view = CalcView;

            //wire-up view events
            view.NumberButtonClicked += new Action<int>(numberButtonClicked);
            view.OperatorButtonClicked += new Action<CalcEnums.Operators>(operatorButtonClicked);
            view.EqualsButtonClicked += new Action(equalsButtonClicked);
            view.ClearButtonClicked += new Action(clearButtonClicked);
            view.DecimalButtonClicked += new Action(decimalButtonClicked);

            model.Reset();

            view.EnableEq(false);
            view.EnableNumberButtons(true);
            view.EnableOpButtons(false);
        }

        #region handle button clicked events
        void numberButtonClicked(int digit)
        {
            string num;
            if (model.Operator == CalcEnums.Operators.None)
            {
                model.AppendNum1Digit(digit);
                // appended a digit. will not be null
                num = ((double)model.Num1).ToString();

                view.EnableEq(false);
                view.EnableNumberButtons(true);
                view.EnableOpButtons(true);
            }
            else
            {
                model.AppendNum2Digit(digit);
                // appended a digit. will not be null
                num = model.GetFormula();

                view.EnableEq(true);
                view.EnableNumberButtons(true);
                view.EnableOpButtons(false);
            }
            view.Display(num);
        }

        void decimalButtonClicked()
        {
            if (model.Operator == CalcEnums.Operators.None)
                model.AppendNum1DecimalPoint();
            else
                model.AppendNum2DecimalPoint();
        }

        void clearButtonClicked()
        {
            model.Reset();
            view.Display("");

            view.EnableEq(false);
            view.EnableNumberButtons(true);
            view.EnableOpButtons(false);
        }

        void equalsButtonClicked()
        {
            try
            {
                double result = model.GetResult();
                view.Display(model.GetFormula() + "=" + result.ToString());
            }
            catch
            {
                view.DisplayError("ERROR");
            }

            view.EnableEq(false);
            view.EnableNumberButtons(false);
            view.EnableOpButtons(false);
        }

        void operatorButtonClicked(CalcEnums.Operators op)
        {
            model.Operator = op;

            view.EnableEq(false);
            view.EnableNumberButtons(true);
            view.EnableOpButtons(false);

            view.Display(model.GetFormula());
        }
        #endregion
    }
}
